package com.americancoders.ebXML;

import java.util.Vector;
import java.lang.*;

import com.americancoders.ebXML.*;

/**
 * An ebXML Error Document has a root element that consists of:
 * <ul>
 * <li>an ErrorHeader element that identifies the nature and severity of the error, and
 * <li> zero or more ErrorLocation elements, that identify the part(s) of the message(s) that are
 * in error.
 * </ul>
 *<br>
 *<b><i>This module, both source code and documentation, is in the Public
 * Domain, and comes with NO WARRANTY. </i></b>
 *Copyright (C) 2000 American Coders, Ltd. Raleigh NC USA
 *<br>
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the Mozilla Public License
 * as published by the Mozilla Software Foundation
 * http://www.mozilla.org/MPL
 *<br>
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Mozilla Public License for more details.
 *<br>
 *@version 0.21d.001
 *@author Joe McVerry American Coders, Ltd.
 *@see http://www.ebxml.org
 */

 class ebXMLError
 {
   ErrorHeader errorHeader;
   ErrorLocation errorLocation;
   Vector errorLocations = new Vector();


 /**
 * test routine
 */
 public static void main(String args[])
  {
	ebXMLError ebXMLError = new ebXMLError();
    ErrorHeader errorHeader = new ErrorHeader();
    ebXMLError.setErrorHeader(errorHeader);
    String ErrorCode = "test";
    errorHeader.setErrorCode(ErrorCode);
    String Severity = "test";
    errorHeader.setSeverity(Severity);
    Description Description = new Description();
    errorHeader.addDescription(Description);
    String SoftwareDetails = "test";
    errorHeader.setSoftwareDetails(SoftwareDetails);
    ErrorLocation errorLocation = new ErrorLocation();
    ebXMLError.addErrorLocation(errorLocation);
    errorLocation.setID("Test");
    errorLocation.addRefToMessageId("ref");
    errorLocation.setHref("hREF");

    System.out.println("<?xml version =\"1.0\" encoding=\"UTF-8\"?>");
    System.out.println("<!DOCTYPE ebXMLError SYSTEM \"ebXMLError.dtd\">");
	System.out.println(ebXMLError.getXML());
}

 /**
  * method to set ErrorHeader
  * @param ErrorHeader
  */
   public void setErrorHeader(ErrorHeader inErrorHeader) {errorHeader = inErrorHeader;}

 /**
  * method to get ErrorHeader
  * @return errorHeader
  */
   public ErrorHeader getErrorHeader() {return errorHeader;}


 /**
   * method to add ErrorLocations
   * @param ErrorLocation
   */
   public void addErrorLocation(ErrorLocation inErrorLocations) {errorLocations.addElement(inErrorLocations);}

 /**
   * method to get ErrorLocations
   * @returns  Vector
   */
   public Vector getErrorLocations() {return errorLocations;}

  /**
   * method to build an XML string of this and its subordinate objects
   * @param int depth - how many multiple of 4 spaces to add in front of text
   * @return String - XML text
   */
  public String getXML() {
	StringBuffer sb = new StringBuffer();
	int depth = 0;
    sb.append("<ebXMLError>\n");
    if (getErrorHeader() == null)
       System.err.println("errorHeader object missing from ebXMLError object");
    else
       sb.append(getErrorHeader().getXML(depth));
    for (int i = 0; i < errorLocations.size(); i++)
      {
		  errorLocation = (ErrorLocation) errorLocations.elementAt(i);
		  sb.append(errorLocation.getXML(depth));
	  }
    sb.append("</ebXMLError>");
	return new String(sb);
  }
}

/** The ErrorHeader element identifies the nature and severity of the error.
 *It consists of the following attributes/elements.
 *<ul>
 *<li>attribute ID
 *<li>element ErrorCode
 *<li>element Severity
 *<li>element SoftwareDetails
 *</ul>
 */
class ErrorHeader
 {
 String ID;
 String errorCode;
 String severity;
 Description description;
 Vector descriptions = new Vector();
 String softwareDetails;

 /**
   * method to set ID
   * @param String
   */
 public void setID(String inID) {ID = inID;}

 /**
   * method to get ID
   * @returns  String
   */
 public String getID() {return ID;}

 /**
   * method to set ErrorCode
   * @param String
   */
 public void setErrorCode(String inErrorCode) {errorCode = inErrorCode;}

 /**
   * method to get ErrorCode
   * @returns  String
   */
 public String getErrorCode() {return errorCode;}

 /**
   * method to set Severity
   * @param String
   */
 public void setSeverity(String inSeverity) {severity = inSeverity;}

 /**
   * method to get Severity
   * @returns  String
   */
 public String getSeverity() {return severity;}

 /**
   * method to add a Description
   * @param Description
   */
 public void addDescription(Description inDescription) {descriptions.addElement(inDescription);}

 /**
   * method to get Description
   * @returns  Description
   */
 public Description getDescription() {return description;}

 /**
   * method to set SoftwareDetails
   * @param String
   */
 public void setSoftwareDetails(String inSoftwareDetails) {softwareDetails = inSoftwareDetails;}

 /**
   * method to get SoftwareDetails
   * @returns  String
   */
 public String getSoftwareDetails() {return softwareDetails;}

/**
 * method to build an XML string of this and its subordinate objects
 * @param int depth - how many multiple of 4 spaces to add in front of text
 * @return String - XML text
 */
 public String getXML(int depth) {
	StringBuffer sb = new StringBuffer();
	depth++;
    sb.append(ebXMLHeader.addDepth(depth)+"<ErrorHeader>\n");
    if (getID() == null)
       System.err.println("ID object missing from ErrorHeader object");
    else
    {
       sb.append(ebXMLHeader.addDepth(depth+1)+"<ID>");
       sb.append(getID());
	   sb.append("</ID>\n");
    }
    if (getErrorCode() == null)
       System.err.println("ErrorCode object missing from ErrorHeader object");
    else   {
        sb.append(ebXMLHeader.addDepth(depth+1)+"<ErrorCode>");
	    sb.append(getErrorCode());
	    sb.append("</ErrorCode>\n");
	}
	if (getSeverity() == null)
	   System.err.println("Severity object missing from ErrorHeader object");
	else   {
        sb.append(ebXMLHeader.addDepth(depth+1)+"<Severity>");
	    sb.append(getSeverity());
	    sb.append("</Severity>\n");
	}
	for (int i=0; i < descriptions.size(); i++)
	  {
		  description = (Description) descriptions.elementAt(i);
		  sb.append(description.getXML(depth));
	  }
	if (getSoftwareDetails() == null)
	   System.err.println("SoftwareDetails object missing from ErrorHeader object");
	else
	{
      sb.append(ebXMLHeader.addDepth(depth+1)+"<SoftwareDetails>");
	  sb.append(getSoftwareDetails());
	  sb.append("</SoftwareDetails>\n");
    }
	sb.append(ebXMLHeader.addDepth(depth)+"</ErrorHeader>\n");
	return new String(sb);
  }
}

/**
 * The optional Description element provides a narrative description of the error in the language defined by the xml:lang attribute on the Description element. The content of this attribute is
 * defined by the vendor/developer of the software, which generated the ebXML Error Document.
 * xml:lang must comply with the rules for identifying languages specified in [XML].
 */

class Description {
   String xmllang = "en";
   String value;

 /**
   * method to set XMLLang
   * @param String
   */
 public void setXMLLang(String inXMLLang) {xmllang = inXMLLang;}

   /**
     * method to get XMLLang
     * @returns  String
     */
 public String getXMLLang() {return xmllang;}

 /**
   * method to set Value
   * @param String
   */
 public void setValue(String inValue) {value = inValue;}

   /**
     * method to get Value
     * @returns  String
     */
 public String getValue() {return value;}

/**
 * method to build an XML string of this and its subordinate objects
 * @param int depth - how many multiple of 4 spaces to add in front of text
 * @return String - XML text
 */
 public String getXML(int depth) {
	StringBuffer sb = new StringBuffer();
	depth++;
    sb.append(ebXMLHeader.addDepth(depth)+"<Description");
    sb.append("  xml:lang=\""+getXMLLang()+"\">");
    if (getValue() == null)
      System.err.println("value object missing from Description object");
    else
      sb.append(getValue());
	sb.append("</Description>\n");
	return new String(sb);
  }
}


/**
 * The ErrorLocation Element identifies the location of an error either within a message or elsewhere.
 * <br>
 * Frequently a single ErrorLocation element will be all that is required within an ebXML Error
 * document. For example, an ErrorCode of ValueNotRecognized is likely to reference an element
 * or attribute and no other ErrorLocation element will be needed.
 */
class ErrorLocation
 {
  String ID = "";
  String refToMessageId;
  Vector refToMessageIds = new Vector();
  String href;

 /**
   * method to set ID
   * @param String
   */
  public void setID(String inID) {ID = inID;}

  /**
    * method to get ID
    * @returns  String
    */
  public String getID() {return ID;}

 /**
   * method to add RefToMessageId
   * @param String
   */
  public void addRefToMessageId(String inRefToMessageId) {refToMessageIds.addElement(inRefToMessageId);}

 /**
   * method to get RefToMessageId
   * @returns  Vector
   */
  public Vector getRefToMessageIds() {return refToMessageIds;}

 /**
   * method to set Href
   * @param String
   */
  public void setHref(String inHref) {href = inHref;}

  /**
    * method to get Href
    * @returns  String
    */
  public String getHref() {return href;}

 /**
  * method to build an XML string of this and its subordinate objects
  * @param int depth - how many multiple of 4 spaces to add in front of text
  * @return String - XML text
  */
  public String getXML(int depth) {
	StringBuffer sb = new StringBuffer();
	depth++;
    sb.append(ebXMLHeader.addDepth(depth)+"<ErrorLocation");
    sb.append(" ID=\""+getID()+"\">\n");
    for (int i=0; i<refToMessageIds.size();i++)
      {
          sb.append(ebXMLHeader.addDepth(depth+1)+"<RefToMessageId>");
		  sb.append((String) refToMessageIds.elementAt(i));
		  sb.append("</RefToMessageId>\n");
	  }
    if (getHref() == null)
	      System.err.println("href object missing from Description object");
	else {
       sb.append(ebXMLHeader.addDepth(depth+1)+"<Href>");
   	   sb.append(href);
	   sb.append("</Href>\n");
    }
	sb.append(ebXMLHeader.addDepth(depth)+"</ErrorLocation>\n");
	return new String(sb);
 }
}
