package com.americancoders.ebXML;

import org.xml.sax.*;
import java.util.*;
import java.io.*;

import com.americancoders.ebXML.*;

/** class to parse an ebXMLHeader.
 * <br>stores all objects as defined in ebXMLError
 * <br>requires xerces SAX parser <a href=http://www.apache.org>From the apache group</a>
 * <br>
 * <br> package built from ebXML Messaging Service Specification
 * <br>ebXML Transport, Routing & Packaging
 * <br> Version 0.21d
 * <br> 16 October 2000
 * <br>Note this specification is in development and is subject to change
 * <br>
 * <b><i>This module, both source code and documentation, is in the Public
 * Domain, and comes with NO WARRANTY. </i></b>
 *<br>
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the Mozilla Public License
 * as published by the Mozilla Software Foundation
 * http://www.mozilla.org/MPL
 *<br>
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Mozilla Public License for more details.
 *<br>
 * @version 0.21d.001
 * @author Joe McVerry American Coders, Ltd.
 * @see http://www.ebxml.org
 */
public class ebXMLErrorParser extends org.xml.sax.helpers.DefaultHandler
{
  protected int		  _iElement = 0;
  protected int		  _iLine = 0;
  protected Stack objectStack = new Stack();
  protected Object currentObject;
  protected org.apache.xerces.parsers.SAXParser parser;
  private CharArrayWriter contents = new CharArrayWriter();

  protected ebXMLError ebXMLError ;
  protected ErrorHeader errorHeader;
  protected String errorCode;
  protected String severity;
  protected Description description;
  protected String softwareDetails;
  protected ErrorLocation errorLocation;
  protected String refToMessageId;
  protected String href;

  /**
   * construct the parser object from xerces
   * -exception SAXEception is caught for setting features
   * -exception generic Java exceptions are caught and stack is dumped to System.out
   */
  public ebXMLErrorParser() {
    try {
      parser = new org.apache.xerces.parsers.SAXParser();
      try {
        parser.setFeature("http://xml.org/sax/features/validation", true);
        parser.setFeature("http://xml.org/sax/features/namespaces", false);
      }
      catch (SAXException e)
      {System.out.println("error in setting up parser feature");}
      parser.setContentHandler(this);
      parser.setErrorHandler(this);
      }
      catch (Exception e1)  {e1.printStackTrace();}
  }


/**
 * test routine  pass one arg - filename of file containing xml document
 */
public static void main(String args[])
{
	if (args.length < 1)
	  {
		  System.err.println("format: java com.americancoders.ebXML.ebXMLErrorParser filename");
		  System.exit(0);
	  }
    ebXMLErrorParser exep = null;
	try {
        	exep = new ebXMLErrorParser();
            InputSource is = new InputSource("file:///"+args[0]);
   	        exep.parse(is);
		}
	catch (Exception e) {
		e.printStackTrace();
   	    System.exit(0);
	}
	System.out.println(exep.getebXMLError().getXML());
}

  /**
   * method parse a XML String
   * @param String contains XML text
   * -exception generic Java exceptions are caught and stack is dumped to System.out
   */

  public void parse(String xmlString)
  {
	 try {
      InputSource is = new InputSource(new CharArrayReader(xmlString.toCharArray()));
      parser.parse(is);
    } /* endtry */
    catch ( Exception e1 ) {
      e1.printStackTrace();

    } /* endcatch */
  }

  /**
   * method parse a file
   * @param Java.io.File file containing XML text
   * -exception generic Java exceptions are caught and stack is dumped to System.out
   */

  public void parse(File xmlFile)
  {
	 try {
      InputSource is = new InputSource(new FileInputStream(xmlFile));
      parser.parse(is);
    } /* endtry */
    catch ( Exception e1 ) {
      e1.printStackTrace();

    } /* endcatch */
  }

  /**
   * method parse an InputSource
   * @param InputSource
   * -exception generic Java exceptions are caught and stack is dumped to System.out
   */

  public void parse(InputSource is)
  {
	 try {
      parser.parse(is);
    } /* endtry */
    catch ( Exception e1 ) {
      e1.printStackTrace();

    } /* endcatch */
  }

  /**
   * method to get the ebXMLError that was parsed.
   * <br>you will need to get the ebXMLError to get access to the child objects
   * @return ebXHMLheader
   */
  public ebXMLError getebXMLError() { return ebXMLError; }

  /**
   * method called for each xml element found.
   * <br> process logic
   * <ul>
   * <li> based on the name of the element found
   * <li> for each pull appropriate attributes and construct object
   * </ul>
   * @param String data element name
   * @param AttributeList attributes associated with data element
   *
   */
  public void startElement(java.lang.String uri,
  java.lang.String localName,
  java.lang.String rawName,
  Attributes attributes)
  throws SAXException
  {

    int i;

    _iElement++;

    contents.reset();

    if (rawName.compareTo("ebXMLError") == 0) {
	  ebXMLError = new ebXMLError();
	  currentObject = ebXMLError;
    }
    else
    if (rawName.compareTo("ErrorHeader") == 0) {
       errorHeader = new ErrorHeader();
       if (attributes.getValue("ID") != null)
         {
			 errorHeader.setID(attributes.getValue("ID"));
		 }
	   ebXMLError.setErrorHeader(errorHeader);
    }
    else
    if (rawName.compareTo("ErrorCode") == 0) {
	  errorCode = "NotSetYet";
	  errorHeader.setErrorCode(errorCode);
	  currentObject = errorCode;
    }
    else
    if (rawName.compareTo("Severity") == 0) {
	  severity = "NotSetYet";
	  errorHeader.setSeverity(severity);
	  currentObject = severity;
    }
    else if (rawName.compareTo("Description") == 0) {
      description = new Description();
      errorHeader.addDescription(description);
      if (attributes.getValue("xml:lang") != null)
         {
			 description.setXMLLang(attributes.getValue("xml:lang"));
		 }
	}
    else
    if (rawName.compareTo("SoftwareDetails") == 0) {
	  softwareDetails = "NotSetYet";
	  errorHeader.setSoftwareDetails(softwareDetails);
	  currentObject = softwareDetails;
    }
    else if (rawName.compareTo("ErrorLocation") == 0) {
       errorLocation = new ErrorLocation();
       ebXMLError.addErrorLocation(errorLocation);
       if (attributes.getValue("ID") != null)
         {
			 errorLocation.setID(attributes.getValue("ID"));
		 }
    }
    else
    if (rawName.compareTo("RefToMessageId") == 0) {
	  refToMessageId = "NotSetYet";
	  currentObject = refToMessageId;
    }
    else
    if (rawName.compareTo("Href") == 0) {
	  href = "NotSetYet";
	  currentObject = href;
    }
    else System.err.println("Logic error: Unknown element name \"" + rawName
    + "\" found during startElement processing at element position " + _iElement + " line: " + _iLine);

    objectStack.push(currentObject);
  }


  /**
   * Method declaration
   *
   *
   * @param ch
   * @param start
   * @param length
   *
   *
   */
  public void characters(char ch[], int start, int length)
  throws SAXException {
    contents.write( ch, start, length );
  }

  /**
   * Method called by the SAX parser at the </
   *
   *
   * @param String name of element found
   *
   * @throws SAXException
   *
   *
   */
  public void endElement(java.lang.String uri,
  java.lang.String localName,
  java.lang.String rawName)
  throws SAXException{

    currentObject = objectStack.pop();

    if (rawName.compareTo("ebXMLError") == 0) {
    }
	else
    if (rawName.compareTo("ErrorHeader") == 0) {
    }
	else
    if (rawName.compareTo("ErrorCode") == 0) {
		errorHeader.setErrorCode(contents.toString());
    }
	else
    if (rawName.compareTo("Severity") == 0) {
		errorHeader.setSeverity(contents.toString());
    }
	else
    if (rawName.compareTo("Description") == 0) {
		description.setValue(contents.toString());
    }
	else
    if (rawName.compareTo("SoftwareDetails") == 0) {
		errorHeader.setSoftwareDetails(contents.toString());
    }
	else
    if (rawName.compareTo("ErrorLocation") == 0) {
    }
    else
    if (rawName.compareTo("RefToMessageId") == 0) {
		errorLocation.addRefToMessageId(contents.toString());
    }
	else
    if (rawName.compareTo("Href") == 0) {
		errorLocation.setHref(contents.toString());
    }
    else System.err.println("Logic error: Unknown element name \"" + rawName
    + "\" found during endElement processing at element position " + _iElement + " line: " + _iLine);


  }


  /**
   * override the SAX2 ignorableWhitespace method to keep track of line numbers
   */
  public void ignorableWhitespace(char[] ch, int start, int length)
  {
    for (int i=start; i < start+length; i++)
    if (ch[i] == '\n') _iLine++;
  }

  /**
   * catches warning SAXParseExceptions
   * this code sends exception to stdio and allows public classto continue
   */
  public void warning(SAXParseException e) throws SAXException {
    System.err.println("Warning at (file " + e.getSystemId() + ", line "
    + e.getLineNumber() + ", char "
    + e.getColumnNumber() +"): " + e.getMessage());
  }

  /**
   * catches error SAXParseExceptions
   * this code causes exception to continue
   */
  public void error(SAXParseException e) throws SAXException {
    throw new SAXException("Error at (file " + e.getSystemId()
    + ", line " + e.getLineNumber() + ", char "
    + e.getColumnNumber() +"): "
    + e.getMessage());
  }

  /**
   * catches fatal SAXParseExceptions
   * this code causes exception to continue
   */
  public void fatalError(SAXParseException e) throws SAXException {
    throw new SAXException("Fatal Error at (file " + e.getSystemId()
    + ", line " + e.getLineNumber() + ", char "
    + e.getColumnNumber() +"): "
    + e.getMessage());
  }


}

