package com.americancoders.ebXML;

import org.xml.sax.*;
import java.util.*;
import java.io.*;

import com.americancoders.ebXML.*;

/** class to parse an ebXMLHeader.
 * <br>stores all objects as defined in ebXMLHeader
 * <br>requires xerces SAX parser <a href=http://www.apache.org>From the apache group</a>
 * <br>
 * <br> package built from ebXML Messaging Service Specification
 * <br>ebXML Transport, Routing & Packaging
 * <br> Version 0.21d
 * <br> 16 October 2000
 * <br>Note this specification is in development and is subject to change
 * <br>
 * <b><i>This module, both source code and documentation, is in the Public
 * Domain, and comes with NO WARRANTY. </i></b>
 *<br>
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the Mozilla Public License
 * as published by the Mozilla Software Foundation
 * http://www.mozilla.org/MPL
 *<br>
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Mozilla Public License for more details.
 *<br>
 * @version 0.21d.001
 * @author Joe McVerry American Coders, Ltd.
 * @see http://www.ebxml.org
 */
public class ebXMLHeaderParser extends org.xml.sax.helpers.DefaultHandler
{
  protected int		  _iElement = 0;
  protected int		  _iLine = 0;
  protected Stack objectStack = new Stack();
  protected Object currentObject;
  protected org.apache.xerces.parsers.SAXParser parser;
  private CharArrayWriter contents = new CharArrayWriter();

  protected ebXMLHeader ebXMLHeader ;
  protected Manifest manifest;
  protected DocumentReference documentReference ;
  protected ValueType documentDescription;
  protected ValueType documentLabel ;
  protected ValueType  documentId  ;
  protected Header header ;
  protected TPAInfo tPAInfo;
  protected ValueType serviceInterface;
  protected ValueType action;
  protected ValueTypeContext tPAId ;
  protected ValueTypeContext conversationId;
  protected MessageData messageData ;
  protected ValueType refToMessageId;
  protected ValueType timeStamp ;
  protected ValueType messageId ;
  protected From from;
  protected To to;
  protected ValueTypeContext partyId;
  protected ReliableMessagingInfo reliableMessagingInfo;
  protected RoutingHeader routingHeader;
  protected ValueType senderURI;
  protected ValueType receiverURI;
  protected ValueType errorURI;
  protected String sequenceNumber;

  /**
   * construct the parser object from xerces
   * -exception SAXEception is caught for setting features
   * -exception generic Java exceptions are caught and stack is dumped to System.out
   */
  public ebXMLHeaderParser() {
    try {
      parser = new org.apache.xerces.parsers.SAXParser();
      try {
        parser.setFeature("http://xml.org/sax/features/validation", true);
        parser.setFeature("http://xml.org/sax/features/namespaces", false);
      }
      catch (SAXException e)
      {System.out.println("error in setting up parser feature");}
      parser.setContentHandler(this);
      parser.setErrorHandler(this);
      }
      catch (Exception e1)  {e1.printStackTrace();}
  }


/**
 * test routine  pass one arg - filename of file containing xml document
 */
public static void main(String args[])
{
	if (args.length < 1)
	  {
		  System.err.println("format: java com.americancoders.ebXML.ebXMLHeaderParser filename");
		  System.exit(0);
	  }
    ebXMLHeaderParser exhp = null;
	try {
        	exhp = new ebXMLHeaderParser();
            InputSource is = new InputSource("file:///"+args[0]);
   	        exhp.parse(is);
		}
	catch (Exception e) {
		e.printStackTrace();
   	    System.exit(0);
	}
	System.out.println(exhp.getebXMLHeader().getXML());
}

  /**
   * method parse a XML String
   * @param String contains XML text
   * -exception generic Java exceptions are caught and stack is dumped to System.out
   */

  public void parse(String xmlString)
  {
	 try {
      InputSource is = new InputSource(new CharArrayReader(xmlString.toCharArray()));
      parser.parse(is);
    } /* endtry */
    catch ( Exception e1 ) {
      e1.printStackTrace();

    } /* endcatch */
  }

  /**
   * method parse an InputSource
   * @param InputSource
   * -exception generic Java exceptions are caught and stack is dumped to System.out
   */

  public void parse(InputSource is)
  {
	 try {
      parser.parse(is);
    } /* endtry */
    catch ( Exception e1 ) {
      e1.printStackTrace();

    } /* endcatch */
  }

  /**
   * method parse a file
   * @param Java.io.File file containing XML text
   * -exception generic Java exceptions are caught and stack is dumped to System.out
   */

  public void parse(File xmlFile)
  {
	 try {
      InputSource is = new InputSource(new FileInputStream(xmlFile));
      parser.parse(is);
    } /* endtry */
    catch ( Exception e1 ) {
      e1.printStackTrace();

    } /* endcatch */
  }

  /**
   * method to get the ebXMLHeader that was parsed.
   * <br>you will need to get the ebXMLHeader to get access to the child objects
   * @return ebXHMLheader
   */
  public ebXMLHeader getebXMLHeader() { return ebXMLHeader; }

  /**
   * method called for each xml element found.
   * <br> process logic
   * <ul>
   * <li> based on the name of the element found
   * <li> for each pull appropriate attributes and construct object
   * </ul>
   * @param String data element name
   * @param AttributeList attributes associated with data element
   *
   */
  public void startElement(java.lang.String uri,
  java.lang.String localName,
  java.lang.String rawName,
  Attributes attributes)
  throws SAXException
  {

    int i;

    _iElement++;

    contents.reset();

    if (rawName.compareTo("ebXMLHeader") == 0) {
	  ebXMLHeader = new ebXMLHeader();
      if (attributes.getValue("Version") != null)
         {
			 ebXMLHeader.setVersion(attributes.getValue("Version"));
		 }
      if (attributes.getValue("MessageType") != null)
         {
			 ebXMLHeader.setMessageType(attributes.getValue("MessageType"));
		 }
	  currentObject = ebXMLHeader;
    }
    else
    if (rawName.compareTo("Manifest") == 0) {
	  manifest = new Manifest();
	  ebXMLHeader.setManifest(manifest);
	  currentObject = manifest;
    }
    else
    if (rawName.compareTo("DocumentReference") == 0) {
	  documentReference = new DocumentReference();
	  manifest.addDocumentReference(documentReference);
	  currentObject = documentReference;
    }
    else
    if (rawName.compareTo("DocumentDescription") == 0) {
	  documentDescription = new ValueType();
	  documentReference.addDocumentDescription(documentDescription);
	  currentObject = documentDescription;
    }
    else
    if (rawName.compareTo("DocumentLabel") == 0) {
	  documentLabel = new ValueType();
	  documentReference.setDocumentLabel(documentLabel);
	  currentObject = documentLabel;
    }
    else
    if (rawName.compareTo("DocumentId") == 0) {
	  documentId = new ValueType();
	  documentReference.setDocumentId(documentId);
	  currentObject = documentId;
    }
    else
    if (rawName.compareTo("Header") == 0) {
	  header = new Header();
	  ebXMLHeader.setHeader(header);
	  currentObject = header;
    }
    else
    if (rawName.compareTo("TPAInfo") == 0) {
	  tPAInfo = new TPAInfo();
	  header.setTPAInfo(tPAInfo);
	  currentObject = tPAInfo;
    }
    else
    if (rawName.compareTo("ServiceInterface") == 0) {
	  serviceInterface = new ValueType();
	  tPAInfo.setServiceInterface(serviceInterface);
	  currentObject = serviceInterface;
    }
    else
    if (rawName.compareTo("Action") == 0) {
	  action = new ValueType();
	  tPAInfo.setAction(action);
	  currentObject = action;
    }
    else
    if (rawName.compareTo("TPAId") == 0) {
	  tPAId = new ValueTypeContext();
	  tPAInfo.setTPAId(tPAId);
      if (attributes.getValue("context") != null)
         {
			 tPAId.setContext(attributes.getValue("context"));
		 }
	  currentObject = tPAId;
    }
    else
    if (rawName.compareTo("ConversationId") == 0) {
	  conversationId = new ValueTypeContext();
	  tPAInfo.setConversationId(conversationId);
      if (attributes.getValue("context") != null)
         {
			 conversationId.setContext(attributes.getValue("context"));
		 }
	  currentObject = conversationId;
    }
    else
    if (rawName.compareTo("MessageData") == 0) {
	  messageData = new MessageData();
	  header.setMessageData(messageData);
	  currentObject = messageData;
    }
    else
    if (rawName.compareTo("RefToMessageId") == 0) {
	  refToMessageId = new ValueType();
	  messageData.setRefToMessageId(refToMessageId);
	  currentObject = refToMessageId;
    }
    else
    if (rawName.compareTo("TimeStamp") == 0) {
	  timeStamp = new ValueType();

	  if (currentObject == messageData)
	     messageData.setTimeStamp(timeStamp);
	  else
	  if (currentObject == routingHeader)
	     routingHeader.setTimeStamp(timeStamp);

	  currentObject = timeStamp;
    }
    else
    if (rawName.compareTo("MessageId") == 0) {
	  messageId = new ValueType();
	  messageData.setMessageId(messageId);
	  currentObject = messageId;
    }
    else
    if (rawName.compareTo("From") == 0) {
	  from = new From();
	  header.setFrom(from);
	  currentObject = from;
    }
    else
    if (rawName.compareTo("To") == 0) {
	  to = new To();
	  header.setTo(to);
	  currentObject = to;
    }
    else
    if (rawName.compareTo("PartyId") == 0) {
	  partyId = new ValueTypeContext();
      if (attributes.getValue("context") != null)
         {
			 partyId.setContext(attributes.getValue("context"));
		 }
	  currentObject = partyId;
    }
    else
    if (rawName.compareTo("ReliableMessagingInfo") == 0) {
	  reliableMessagingInfo = new ReliableMessagingInfo();
	  header.setReliableMessagingInfo(reliableMessagingInfo);
      if (attributes.getValue("DeliverySemantics") != null)
         {
			 reliableMessagingInfo.setDeliverySemantics(attributes.getValue("DeliverySemantics"));
		 }
	  currentObject = reliableMessagingInfo;
    }
    else
    if (rawName.compareTo("RoutingHeader") == 0) {
	  routingHeader = new RoutingHeader();
	  ebXMLHeader.setRoutingHeader(routingHeader);
	  currentObject = routingHeader;
    }
    else
    if (rawName.compareTo("SenderURI") == 0) {
	  senderURI = new ValueType();
	  routingHeader.setSenderURI(senderURI);
	  currentObject = senderURI;
    }
    else
    if (rawName.compareTo("ReceiverURI") == 0) {
	  receiverURI = new ValueType();
	  routingHeader.setReceiverURI(receiverURI);
	  currentObject = receiverURI;
    }
    else
    if (rawName.compareTo("ErrorURI") == 0) {
	  errorURI = new ValueType();
	  routingHeader.setErrorURI(errorURI);
	  currentObject = errorURI;
    }
    else
    if (rawName.compareTo("SequenceNumber") == 0) {
	  sequenceNumber = "NotSetYet";
	  currentObject = sequenceNumber;
    }
    else System.err.println("Logic error: Unknown element name \"" + rawName
    + "\" found during startElement processing at element position " + _iElement + " line: " + _iLine);

    objectStack.push(currentObject);
  }


  /**
   * Method declaration
   *
   *
   * @param ch
   * @param start
   * @param length
   *
   *
   */
  public void characters(char ch[], int start, int length)
  throws SAXException {
    contents.write( ch, start, length );
  }

  /**
   * Method called by the SAX parser at the </
   *
   *
   * @param String name of element found
   *
   * @throws SAXException
   *
   *
   */
  public void endElement(java.lang.String uri,
  java.lang.String localName,
  java.lang.String rawName)
  throws SAXException{

    currentObject = objectStack.pop();

    if (rawName.compareTo("ebXMLHeader") == 0) {
    }
	else
    if (rawName.compareTo("Manifest") == 0) {
    }
	else
    if (rawName.compareTo("DocumentReference") == 0) {
    }
	else
    if (rawName.compareTo("DocumentDescription") == 0) {
		documentDescription = (ValueType) currentObject;
		documentDescription.setValue(contents.toString());
    }
	else
    if (rawName.compareTo("DocumentLabel") == 0) {
		documentLabel.setValue(contents.toString());
    }
	else
    if (rawName.compareTo("DocumentId") == 0) {
		documentId.setValue(contents.toString());
    }
	else
    if (rawName.compareTo("Header") == 0) {
    }
	else
    if (rawName.compareTo("TPAInfo") == 0) {
    }
	else
    if (rawName.compareTo("ServiceInterface") == 0) {
		serviceInterface.setValue(contents.toString());
    }
	else
    if (rawName.compareTo("Action") == 0) {
		action.setValue(contents.toString());
    }
	else
    if (rawName.compareTo("TPAId") == 0) {
		tPAId.setValue(contents.toString());
    }
	else
    if (rawName.compareTo("ConversationId") == 0) {
		conversationId.setValue(contents.toString());
    }
	else
    if (rawName.compareTo("MessageData") == 0) {
		messageData.setTimeStamp(timeStamp);
    }
	else
    if (rawName.compareTo("RefToMessageId") == 0) {
		refToMessageId.setValue(contents.toString());
    }
	else
    if (rawName.compareTo("TimeStamp") == 0) {
		timeStamp.setValue(contents.toString());
    }
	else
    if (rawName.compareTo("MessageId") == 0) {
		messageId.setValue(contents.toString());
    }
	else
    if (rawName.compareTo("From") == 0) {
		from.setPartyId(partyId);
    }
	else
    if (rawName.compareTo("To") == 0) {
		to.setPartyId(partyId);
    }
	else
    if (rawName.compareTo("PartyId") == 0) {
		partyId.setValue(contents.toString());
    }
	else
    if (rawName.compareTo("ReliableMessagingInfo") == 0) {
    }
    else
    if (rawName.compareTo("RoutingHeader") == 0) {
		routingHeader.setTimeStamp(timeStamp);
		routingHeader.setSequenceNumber(sequenceNumber); // a little different becuase the String is saved prior to setting
    }
	else
	if (rawName.compareTo("SenderURI") == 0){
		senderURI.setValue(contents.toString());
	}
	else
	if (rawName.compareTo("ReceiverURI") == 0){
		receiverURI.setValue(contents.toString());
	}
	else
	if (rawName.compareTo("ErrorURI") == 0){
		errorURI.setValue(contents.toString());
	}
	else
	if (rawName.compareTo("SequenceNumber") == 0){
		sequenceNumber = contents.toString();
	}
    else System.err.println("Logic error: Unknown element name \"" + rawName
    + "\" found during endElement processing at element position " + _iElement + " line: " + _iLine);


  }


  /**
   * override the SAX2 ignorableWhitespace method to keep track of line numbers
   */
  public void ignorableWhitespace(char[] ch, int start, int length)
  {
    for (int i=start; i < start+length; i++)
    if (ch[i] == '\n') _iLine++;
  }

  /**
   * catches warning SAXParseExceptions
   * this code sends exception to stdio and allows public classto continue
   */
  public void warning(SAXParseException e) throws SAXException {
    System.err.println("Warning at (file " + e.getSystemId() + ", line "
    + e.getLineNumber() + ", char "
    + e.getColumnNumber() +"): " + e.getMessage());
  }

  /**
   * catches error SAXParseExceptions
   * this code causes exception to continue
   */
  public void error(SAXParseException e) throws SAXException {
    throw new SAXException("Error at (file " + e.getSystemId()
    + ", line " + e.getLineNumber() + ", char "
    + e.getColumnNumber() +"): "
    + e.getMessage());
  }

  /**
   * catches fatal SAXParseExceptions
   * this code causes exception to continue
   */
  public void fatalError(SAXParseException e) throws SAXException {
    throw new SAXException("Fatal Error at (file " + e.getSystemId()
    + ", line " + e.getLineNumber() + ", char "
    + e.getColumnNumber() +"): "
    + e.getMessage());
  }


}

